"""
Django settings for edx_sga project.
"""

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)

import os

DEFAULT_FILE_STORAGE = "django.core.files.storage.FileSystemStorage"
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
SECRET_KEY = "k(g$9p*x^a=t_lf$%=b*3$$ipg1g0tm!^pws0@g)u+v@&$657n"
DEBUG = True

ALLOWED_HOSTS = []
INSTALLED_APPS = (
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "workbench",
)
MIDDLEWARE_CLASSES = (
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.auth.middleware.SessionAuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
    "django.middleware.security.SecurityMiddleware",
)
TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ],
        },
    },
]
WSGI_APPLICATION = "edx_sga.wsgi.application"


# Database
DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": os.path.join(BASE_DIR, "db.sqlite3"),
    }
}

# Internationalization
LANGUAGE_CODE = "en-us"
TIME_ZONE = "UTC"
USE_I18N = True
USE_L10N = True
USE_TZ = True
STATIC_URL = "/static/"
TECH_SUPPORT_EMAIL = "support@example.com"
# For xblock-sdk workbench
WORKBENCH = {}



"""
Tests for SGA utility functions
"""

import pytest

import pytz
from edx_sga.tests.common import is_near_now
from edx_sga.utils import is_finalized_submission, utcnow


@pytest.mark.parametrize(
    "submission_data,expected_value",
    [
        ({"answer": {"finalized": True}}, True),
        ({"answer": {"filename": "file.txt"}}, True),
        ({"answer": {}}, True),
        ({"answer": {"finalized": False}}, False),
        ({"answer": None}, False),
        ({}, False),
    ],
)
def test_is_finalized_submission(submission_data, expected_value):
    """Test for is_finalized_submission"""
    assert is_finalized_submission(submission_data) is expected_value


def test_utcnow():
    """
    tznow should return a datetime object in UTC
    """
    now = utcnow()
    assert is_near_now(now)
    assert now.tzinfo.zone == pytz.utc.zone



"""Tests for ShowAnswerXBlockMixin"""

import datetime
from unittest import TestCase, mock

import pytest
import ddt
import pytz
from web_fragments.fragment import Fragment
from workbench.runtime import WorkbenchRuntime
from xblock.core import XBlock
from xblock.fields import DateTime, Integer, String


from edx_sga.constants import ShowAnswer
from edx_sga.showanswer import ShowAnswerXBlockMixin

pytestmark = pytest.mark.django_db


class ShowAnswerXBlock(
    ShowAnswerXBlockMixin, XBlock
):  # pylint: disable=abstract-method
    """
    A basic ShowAnswer XBlock implementation (for use in tests)
    """

    CATEGORY = "showanswer"
    STUDIO_LABEL = "Show Answers"

    color = String(default="red")
    count = Integer(default=42)
    comment = String(default="")
    date = DateTime(default=datetime.datetime(2014, 5, 14, tzinfo=pytz.UTC))
    editable_fields = ("color", "count", "comment", "date")

    def student_view(self, context):  # pylint: disable=unused-argument
        """Just a placeholder"""
        return Fragment()


@ddt.ddt
class TestShowAnswerXBlock(TestCase):
    """
    Unit tests for ShowAnswerXBlockMixin
    """

    def setUp(self):
        super().setUp()

        self.runtime = WorkbenchRuntime()

    def get_root(self):
        """Create an xblock block with a ShowAnswerXBlockMixin"""
        root_id = self.runtime.parse_xml_string("<showanswer />")
        return self.runtime.get_block(root_id)

    @ddt.data(
        *[
            [True, True, True],
            [True, False, False],
            [False, True, True],
            [False, False, True],
        ]
    )
    @ddt.unpack
    @XBlock.register_temp_plugin(ShowAnswerXBlock, "showanswer")
    def test_closed(self, can_attempt, past_due, expected):
        """
        Assert possible values for closed()
        """
        block = self.get_root()
        with mock.patch.object(
            ShowAnswerXBlock,
            "can_attempt",
            return_value=can_attempt,
        ), mock.patch.object(
            ShowAnswerXBlock,
            "is_past_due",
            return_value=past_due,
        ):
            self.assertEqual(block.closed(), expected)

    @XBlock.register_temp_plugin(ShowAnswerXBlock, "showanswer")
    def test_answer_available_no_correctness(self):
        """
        If no correctness is available, the answer is not available
        """
        block = self.get_root()
        with mock.patch.object(
            ShowAnswerXBlock, "correctness_available", return_value=False
        ):
            self.assertFalse(block.answer_available())

    @XBlock.register_temp_plugin(ShowAnswerXBlock, "showanswer")
    def test_answer_available_user_is_staff(self):
        """
        If user is staff and correctness is available, the answer is available
        """
        block = self.get_root()
        self.runtime.user_is_staff = True
        with mock.patch.object(
            ShowAnswerXBlock, "correctness_available", return_value=True
        ), mock.patch.object(
            ShowAnswerXBlock,
            "runtime_user_is_staff",
            return_value=True,
        ):
            self.assertTrue(block.answer_available())

    @ddt.data(
        *[
            ["", {}, False],
            [ShowAnswer.NEVER, {}, False],
            [ShowAnswer.ATTEMPTED, {}, False],
            [ShowAnswer.ATTEMPTED, {"has_attempted": True}, True],
            [ShowAnswer.ANSWERED, {}, False],
            [ShowAnswer.ANSWERED, {"is_correct": True}, True],
            [ShowAnswer.CLOSED, {}, False],
            [ShowAnswer.CLOSED, {"closed": True}, True],
            [ShowAnswer.FINISHED, {}, False],
            [ShowAnswer.FINISHED, {"closed": True}, True],
            [ShowAnswer.FINISHED, {"is_correct": True}, True],
            [ShowAnswer.FINISHED, {"closed": True, "is_correct": True}, True],
            [ShowAnswer.CORRECT_OR_PAST_DUE, {}, False],
            [ShowAnswer.CORRECT_OR_PAST_DUE, {"is_correct": True}, True],
            [ShowAnswer.CORRECT_OR_PAST_DUE, {"is_past_due": True}, True],
            [
                ShowAnswer.CORRECT_OR_PAST_DUE,
                {"is_correct": True, "is_past_due": True},
                True,
            ],
            [ShowAnswer.PAST_DUE, {}, False],
            [ShowAnswer.PAST_DUE, {"is_past_due": True}, True],
            [ShowAnswer.ALWAYS, {}, True],
            ["unexpected", {}, False],
        ]
    )
    @ddt.unpack
    @XBlock.register_temp_plugin(ShowAnswerXBlock, "showanswer")
    def test_answer_available_showanswer(self, showanswer, properties, expected):
        """
        Try different values of showanswer and assert answer_available()
        """
        block = self.get_root()
        block.showanswer = showanswer
        with mock.patch.object(
            ShowAnswerXBlock,
            "correctness_available",
            return_value=True,
        ), mock.patch.object(
            ShowAnswerXBlock,
            "runtime_user_is_staff",
            return_value=False,
        ), mock.patch.object(
            ShowAnswerXBlock,
            "has_attempted",
            return_value=properties.get("has_attempted", False),
        ), mock.patch.object(
            ShowAnswerXBlock,
            "is_correct",
            return_value=properties.get("is_correct", False),
        ), mock.patch.object(
            ShowAnswerXBlock,
            "closed",
            return_value=properties.get("closed", False),
        ), mock.patch.object(
            ShowAnswerXBlock,
            "is_past_due",
            return_value=properties.get("is_past_due", False),
        ):
            self.assertEqual(block.answer_available(), expected)



"""Setup for edx-sga XBlock."""
import os

from setuptools import find_packages, setup

import edx_sga


def package_data(pkg, root_list):
    """Generic function to find package_data for `pkg` under `root`."""
    data = []
    for root in root_list:
        for dirname, _, files in os.walk(os.path.join(pkg, root)):
            for fname in files:
                data.append(os.path.relpath(os.path.join(dirname, fname), pkg))

    return {pkg: data}


setup(
    name="edx-sga",
    version=edx_sga.__version__,
    description="edx-sga Staff Graded Assignment XBlock",
    license="GNU Affero General Public License v3 or later (AGPLv3+)",
    url="https://github.com/mitodl/edx-sga",
    author="MITx",
    zip_safe=False,
    packages=find_packages(),
    include_package_data=True,
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU Affero General Public License v3 or later (AGPLv3+)",
        "Operating System :: OS Independent",
        "Natural Language :: English",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Framework :: Django :: 2.2",
        "Framework :: Django :: 3.0",
        "Framework :: Django :: 3.1",
        "Framework :: Django :: 3.2",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Education",
    ],
    install_requires=[
        "XBlock",
        "xblock-utils",
        "web_fragments",
    ],
    entry_points={
        "xblock.v1": [
            "edx_sga = edx_sga.sga:StaffGradedAssignmentXBlock",
        ]
    },
    package_data=package_data("edx_sga", ["static", "templates"]),
)
